LocalVersion

Enum LocalVersion 

pub enum LocalVersion {
    Segments(Vec<LocalSegment>),
    Max,
}
Either a sequence of local segments or [LocalVersion::Sentinel], an internal-only value that compares greater than all other local versions.

Variants



Segments(Vec<LocalSegment>)

A sequence of local segments.



Max

An internal-only value that compares greater to all other local versions.

Implementations

Source

impl LocalVersion

pub fn empty() -> Self

Return an empty local version.

pub fn is_empty(&self) -> bool

Returns true if the local version is empty.

pub fn as_slice(&self) -> LocalVersionSlice<'_>

Convert the local version segments into a slice.

pub fn clear(&mut self)

Clear the local version segments, if they exist.