pub struct VersionPattern { /* private fields */ }Expand description
A version number pattern.
A version pattern appears in a
VersionSpecifier. It is just like a version,
except that it permits a trailing * (wildcard) at the end of the version
number. The wildcard indicates that any version with the same prefix should
match.
A VersionPattern cannot do any matching itself. Instead,
it needs to be paired with an Operator to create a
VersionSpecifier.
Here are some valid and invalid examples:
1.2.3-> verbatim pattern1.2.3.*-> wildcard pattern1.2.*.4-> invalid1.0-dev1.*-> invalid
Implementations
Sourceimpl VersionPattern
impl VersionPattern
Sourcepub fn verbatim(version: Version) -> Self
pub fn verbatim(version: Version) -> Self
Creates a new verbatim version pattern that matches the given version exactly.
Sourcepub fn wildcard(version: Version) -> Self
pub fn wildcard(version: Version) -> Self
Creates a new wildcard version pattern that matches any version with the given version as a prefix.
Sourcepub fn into_version(self) -> Version
pub fn into_version(self) -> Version
Consumes this pattern and returns ownership of the underlying version.